@@ -28,7 +28,7 @@ ph is a command line tool for streaming csv data.
 If you have a csv file `a.csv`, you can pipe it through `ph` on the
 command line by using
 
-$ cat a.csv | ph columns x y | ph tabulate
+$ cat a.csv | ph columns x y | ph eval "z = x**2 - y" | ph show
 
 Use ph help [command] for help on the individual commands.
 
@@ -44,7 +44,7 @@ READERS = {
     "fwf": lambda fname: pd.read_fwf(fname),
     "json": lambda fname: pd.read_json(fname),
     "html": lambda fname: pd.read_html(fname),
-    "clipboard": lambda fname: pd.read_clipboard(fname),
+    "clipboard": lambda: pd.read_clipboard(),
     "excel": lambda fname, sheet: pd.read_excel(fname, sheet),
     "xls": lambda fname, sheet: pd.read_excel(fname, sheet),
     "odf": lambda fname, sheet: pd.read_excel(fname, sheet),
@@ -301,12 +301,15 @@ def astype(type, column=None, newcolumn=None):
 
     """
     df = pipein()
-    if column is None:
-        df = df.astype(type)
-    elif newcolumn is not None:
-        df[newcolumn] = df[column].astype(type)
-    else:
-        df[column] = df[column].astype(type)
+    try:
+        if column is None:
+            df = df.astype(type)
+        elif newcolumn is not None:
+            df[newcolumn] = df[column].astype(type)
+        else:
+            df[column] = df[column].astype(type)
+    except ValueError as err:
+        exit("Could not convert to {}: {}".format(type, err))
     pipeout(df)
 
 
@@ -526,6 +529,17 @@ def to(ftype, fname=None):
     if ftype == "hdf5":
         exit("hdf5 writer not implemented")
 
+    if ftype == "fwf":
+        # pandas has not yet implemented to_fwf
+        df = pipein()
+        content = tabulate_(df.values.tolist(), list(df.columns), tablefmt="plain")
+        if fname:
+            with open(fname, "w") as wout:
+                wout.write(content)
+        else:
+            print(content)
+        exit()
+
     writer = WRITERS[ftype]
     df = pipein()
     fn = getattr(df, writer)
@@ -550,16 +564,53 @@ def from_(ftype="csv"):
     cat a.csv | ph to json | ph from json
 
     """
-    pipeout(pipein(ftype))
+    if ftype == "clipboard":
+        pipeout(READERS["clipboard"]())
+    else:
+        pipeout(pipein(ftype))
 
 
 @register
-def cat(fname=None):
-    """Opens a file if provided (like `open`), or else cats it."""
-    if fname is None:
+def cat(*fnames, axis="index"):
+    """Concatenates all files provided.
+
+    Usage: ph cat a.csv b.csv c.csv
+           ph cat a.csv b.csv c.csv --axis=index  # default
+           ph cat a.csv b.csv c.csv --axis=columns
+
+    If no arguments are provided, read from std in.
+
+    """
+    if axis not in ("index", "columns"):
+        exit("Unknown axis command '{}'".format(axis))
+    if not fnames:
         pipeout(pipein())
     else:
-        pipeout(pd.read_csv(fname))
+        dfs = []
+        for fname in fnames:
+            df = pd.read_csv(fname)
+            dfs.append(df)
+        retval = pd.concat(dfs, axis=axis)
+        pipeout(retval)
+
+
+@register
+def merge(fname1, fname2, how="inner", on=None):
+    """
+    Merging two csv files.
+
+    Usage: ph merge a.csv b.csv --on=ijk
+
+    """
+    hows = ("left", "right", "outer", "inner")
+    if how not in hows:
+        exit("Unknown merge --how={}, must be one of {}".format(how, hows))
+    df1 = pd.read_csv(fname1)
+    df2 = pd.read_csv(fname2)
+    if on is None:
+        pipeout(pd.merge(df1, df2, how=how))
+    else:
+        pipeout(pd.merge(df1, df2, how=how, on=on))
 
 
 @register
@@ -611,13 +662,21 @@ def show(noindex=False):
         tabulate("--headers")
 
 
+def _print_commands(cmds):
+    num_cols = 72 // max(len(cmd) for cmd in cmds)
+    while (len(cmds) % num_cols) != 0:
+        cmds.append("")
+    df = pd.DataFrame(pd.Series(cmds).values.reshape(num_cols, -1))
+    print(tabulate_(df.transpose(), showindex=False))
+
+
 @register
 def help(*args, **kwargs):
     """Writes help (docstring) about the different commands."""
     if not args:
         print("Usage: ph command arguments")
         print(USAGE_TEXT)
-        print("       commands = {}".format(list(COMMANDS.keys())))
+        _print_commands(sorted(COMMANDS.keys()))
         exit(0)
     cmd = args[0]
     import ph
@@ -637,7 +696,7 @@ def help(*args, **kwargs):
 
 
 @registerx("open")
-def open_(ftype, fname, sheet=0):
+def open_(ftype, fname=None, sheet=0):
     """Use a reader to open a file.
 
     Open ftype file with name fname and stream out.
@@ -659,8 +718,17 @@ def open_(ftype, fname, sheet=0):
     kwargs = {}
     if ftype in ("excel", "xls", "odf"):
         kwargs["sheet"] = __tryparse(sheet)
+
+    if ftype == "clipboard" and fname is not None:
+        exit("clipboard does not take fname")
+    if ftype != "clipboard" and fname is None:
+        exit("filename is required for {}".format(ftype))
+
     try:
-        df = reader(fname, **kwargs)
+        if ftype == "clipboard":
+            df = reader(**kwargs)
+        else:
+            df = reader(fname, **kwargs)
     except AttributeError as err:
         exit("{} is not supported in your Pandas installation\n{}".format(ftype, err))
     except ImportError as err:
@@ -804,6 +872,60 @@ def empty():
     print("empty\n{}".format(pipein().empty))
 
 
+@register
+def index():
+    """Reset the index to a 0..n-1 counter.
+
+    Usage: cat a.csv | ph index
+
+    Adds a left-most column `index`.
+    """
+    pipeout(pipein().reset_index())
+
+
+@register
+def sort(col):
+    """Sort csv input by column.
+
+    Usage: cat iris.csv | ph sort setosa
+
+    """
+    df = pipein()
+    if not col in df.columns:
+        exit("Unknown column {}".format(col))
+    pipeout(df.sort_values(col))
+
+
+@register
+def polyfit(x, y, deg=1):
+    """Perform linear/polynomial regression.
+
+    Usage: cat a.csv | ph polyfix x y
+           cat a.csv | ph polyfix x y --deg=1  # default
+           cat a.csv | ph polyfix x y --deg=2  # default
+
+    Outputs a column polyfit_{deg} containing the evaluated index.
+
+    """
+    df = pipein()
+    if x not in df.columns:
+        exit("Unknown column x={}".format(x))
+    if y not in df.columns:
+        exit("Unknown column y={}".format(y))
+    deg = __tryparse(deg)
+    if not isinstance(deg, int) or deg <= 0:
+        exit("deg={} should be a positive int".format(deg))
+    try:
+        import numpy
+    except ImportError:
+        exit("numpy needed for polyfit.  pip install numpy")
+    polynomial = numpy.polyfit(df[x], df[y], deg=deg)
+    f = numpy.poly1d(polynomial)
+
+    df["polyfit_{}".format(deg)] = df[x].apply(f)
+    pipeout(df)
+
+
 pandas_computations = [
     "abs",
     "all",
